import * as React from 'react';
/**
 * Props contract for themed components.
 */
export interface IThemedComponent<TTheme> {
    theme: TTheme;
}
/**
 * Styles can be a function or an object taking in TViewProps for processing.
 */
export declare type IStyleFunction<TViewProps, TStyleSet> = (props: TViewProps) => Partial<TStyleSet>;
export declare type IStylesProp<TViewProps, TStyleSet> = IStyleFunction<TViewProps, TStyleSet> | Partial<TStyleSet>;
/**
 * Foundation interface for styleable components.
 */
export interface IStyleableComponent<TProps, TStyleSet, TTheme> {
    styles?: IStylesProp<TProps, TStyleSet>;
    theme?: TTheme;
}
export declare type IStyleableComponentProps<TProps, TStyleSet, TTheme> = TProps & IStyleableComponent<TProps, TStyleSet, TTheme>;
/**
 * Enforce props contract on state components, including the view prop and its shape.
 */
export declare type IStateComponentProps<TComponentProps, TViewProps> = TComponentProps & {
    renderView: (props: TViewProps) => JSX.Element;
};
/**
 * Imposed state component props contract with styling props as well as a renderView
 * prop that the StateComponent should make use of in its render output (and should be its only render output.)
 */
export declare type IStateComponent<TComponentProps, TViewProps extends IViewComponent<TViewProps, TProcessedStyleSet>, TProcessedStyleSet> = React.ComponentType<IStateComponentProps<TComponentProps, TViewProps>>;
/**
 * The extended view interface provided to views, including the component's children
 * and processed style.
 */
export declare type IViewComponent<TViewProps, TProcessedStyleSet> = React.Props<TViewProps> & {
    classNames: TProcessedStyleSet;
};
export declare type IViewComponentProps<TViewProps, TProcessedStyledSet> = TViewProps & IViewComponent<TViewProps, TProcessedStyledSet>;
/**
 * Component options used by foundation to tie elements together.
 * @param {IComponentOptions} displayName Display name to identify component in React hierarchy.
 * @param {IStylesProp<TViewProps, TStyleSet>} styles Styles prop to pass into component.
 * @param {IStateComponent} view Functional React view component.
 * @param {TStatics} statics Optional static object to pass into constructed component.
 */
export interface IComponentOptions<TViewProps, TStyleSet, TProcessedStyledSet, TTheme, TStatics> {
    displayName: string;
    styles: IStylesProp<TViewProps, TStyleSet>;
    view: (props: IViewComponentProps<TViewProps, TProcessedStyledSet>) => JSX.Element;
    statics?: TStatics;
}
/**
 * Providers used by createComponent to process and apply styling.
 */
export interface IStylingProviders<TViewProps, TStyleSet, TProcessedStyleSet, TContext, TTheme> {
    mergeStyleSets: (...styles: (Partial<TStyleSet> | undefined)[]) => TProcessedStyleSet;
    getCustomizations: (scope: string, context: TContext) => IStyleableComponent<TViewProps, TStyleSet, TTheme>;
    CustomizableContextTypes: any;
}
/**
 * Assembles a higher order component based on the following: styles, theme, view, and state.
 * Imposes a separation of concern and centralizes styling processing to increase ease of use and robustness
 * in how components use and apply styling and theming.
 *
 * Automatically merges and applies themes and styles with theme / styleprops having the highest priority.
 * State component, if provided, is passed in props for processing. Props from state / user are automatically processed
 * and styled before finally being passed to view.
 *
 * State components should contain all stateful behavior and should not generate any JSX, but rather simply call the view prop.
 * Views should simply be stateless pure functions that receive all props needed for rendering their output.
 * State component is optional. If state not provided, created component is essentially a functional stateless component.
 *
 * TComponentProps: A styleable props interface for the created component.
 * TViewProps: The props specific to the view, including processed properties outputted by optional state component. If state
 * component is not provided, TComponentProps is the same as TViewProps.
 * TStyleSet: The type for styles properties.
 * TProcessedStyleSet: The type provided by mergeStyleSets provider after processing TStyleSet and provided to views.
 * TTheme: The type for theme properties as well as the getTheme provider.
 *
 * @param {IComponentOptions} options
 * @param {IStylingProviders} providers
 * @param {IStateComponent} StateComponent
 *
 * If your package has common types for any of the type arguments, such as TTheme and TProcessedStyleSet, it is strongly
 * recommended to make an interface file for your package that reduces the number of types individual components need
 * to provide. For example:
 * @example
 * export type IViewProps<TProps, TStyleSet extends IStyleSet<TStyleSet>> = IViewProps<TProps, IProcessedStyleSet<TStyleSet>>;
 * export type IStyleableComponent<TProps, TStyleSet> = IStyleableComponent<TProps, TStyleSet, ITheme>;
 *
 */
export declare function createComponent<TComponentProps extends IStyleableComponent<TViewProps, TStyleSet, TTheme>, TViewProps, TStyleSet, TProcessedStyleSet, TContext, TTheme, TStatics>(options: IComponentOptions<TViewProps, TStyleSet, TProcessedStyleSet, TTheme, TStatics>, providers: IStylingProviders<TViewProps, TStyleSet, TProcessedStyleSet, TContext, TTheme>, StateComponent: IStateComponent<TComponentProps, TViewProps & IViewComponent<TViewProps, TProcessedStyleSet>, TProcessedStyleSet>): React.StatelessComponent<TComponentProps> & TStatics;
/**
 * This is essentially the same as createComponent. The primary differences are that TComponentProps and TViewProps
 * are equivalent and there is no state component argument.
 *
 * @see {@link createComponent} for more information.
 */
export declare function createStatelessComponent<TComponentProps extends IStyleableComponent<TComponentProps, TStyleSet, TTheme>, TStyleSet, TProcessedStyleSet, TContext, TTheme, TStatics>(options: IComponentOptions<TComponentProps, TStyleSet, TProcessedStyleSet, TTheme, TStatics>, providers: IStylingProviders<TComponentProps, TStyleSet, TProcessedStyleSet, TContext, TTheme>): React.StatelessComponent<TComponentProps> & TStatics;
