import * as tslib_1 from "tslib";
import * as React from 'react';
/**
 * Assembles a higher order component based on the following: styles, theme, view, and state.
 * Imposes a separation of concern and centralizes styling processing to increase ease of use and robustness
 * in how components use and apply styling and theming.
 *
 * Automatically merges and applies themes and styles with theme / styleprops having the highest priority.
 * State component, if provided, is passed in props for processing. Props from state / user are automatically processed
 * and styled before finally being passed to view.
 *
 * State components should contain all stateful behavior and should not generate any JSX, but rather simply call the view prop.
 * Views should simply be stateless pure functions that receive all props needed for rendering their output.
 * State component is optional. If state not provided, created component is essentially a functional stateless component.
 *
 * TComponentProps: A styleable props interface for the created component.
 * TViewProps: The props specific to the view, including processed properties outputted by optional state component. If state
 * component is not provided, TComponentProps is the same as TViewProps.
 * TStyleSet: The type for styles properties.
 * TProcessedStyleSet: The type provided by mergeStyleSets provider after processing TStyleSet and provided to views.
 * TTheme: The type for theme properties as well as the getTheme provider.
 *
 * @param {IComponentOptions} options
 * @param {IStylingProviders} providers
 * @param {IStateComponent} StateComponent
 *
 * If your package has common types for any of the type arguments, such as TTheme and TProcessedStyleSet, it is strongly
 * recommended to make an interface file for your package that reduces the number of types individual components need
 * to provide. For example:
 * @example
 * export type IViewProps<TProps, TStyleSet extends IStyleSet<TStyleSet>> = IViewProps<TProps, IProcessedStyleSet<TStyleSet>>;
 * export type IStyleableComponent<TProps, TStyleSet> = IStyleableComponent<TProps, TStyleSet, ITheme>;
 *
 */
// TODO: Combine these functions into one once conditional types (TS 2.8) can be used. This will allow us to define
//        TComponentProps as being the same as TViewProps when StateComponent is not provided.
// TODO: use theming prop when provided and reconcile with global theme
// TODO: should userProps have higher priority over processedProps? or vice versa? tradeoff between styles priority and controlled values
//        if user props take priority, this could cause some subtle issues like overriding callbacks that state component uses.
// TODO: remove requirement of IStyleableComponent
export function createComponent(options, providers, StateComponent) {
    var result = function (userProps, context) {
        var settings = providers.getCustomizations(options.displayName, context);
        var contextStyles = settings.styles, rest = tslib_1.__rest(settings, ["styles"]);
        var content = function (processedProps) {
            // The approach here is to allow state components to provide only the props they care about, automatically
            //    merging user props and processed props together. This ensures all props are passed properly to view,
            //    including children and styles.
            var propStyles = processedProps.styles || userProps.styles;
            var themedProps = Object.assign({}, rest, userProps, processedProps);
            var viewProps = Object.assign({}, processedProps, userProps, {
                classNames: providers.mergeStyleSets(_evaluateStyle(themedProps, options.styles), _evaluateStyle(themedProps, contextStyles), _evaluateStyle(themedProps, propStyles))
            });
            // TODO: consider rendering view as JSX component with display name in debug mode to aid in debugging
            return options.view(viewProps);
        };
        return React.createElement(StateComponent, tslib_1.__assign({}, userProps, { renderView: content }));
    };
    result.contextTypes = providers.CustomizableContextTypes;
    result.displayName = options.displayName;
    Object.assign(result, options.statics);
    // Later versions of TypeSript should allow us to merge objects in a type safe way and avoid this cast.
    return result;
}
/**
 * This is essentially the same as createComponent. The primary differences are that TComponentProps and TViewProps
 * are equivalent and there is no state component argument.
 *
 * @see {@link createComponent} for more information.
 */
export function createStatelessComponent(options, providers) {
    var result = function (userProps, context) {
        var settings = providers.getCustomizations(options.displayName, context);
        var contextStyles = settings.styles, rest = tslib_1.__rest(settings, ["styles"]);
        var content = function (processedProps) {
            var propStyles = processedProps.styles;
            var themedProps = Object.assign({}, rest, processedProps);
            var viewProps = Object.assign({}, processedProps, {
                classNames: providers.mergeStyleSets(_evaluateStyle(themedProps, options.styles), _evaluateStyle(themedProps, contextStyles), _evaluateStyle(themedProps, propStyles))
            });
            // TODO: consider rendering view as JSX component with display name in debug mode to aid in debugging
            return options.view(viewProps);
        };
        return content(userProps);
    };
    result.contextTypes = providers.CustomizableContextTypes;
    result.displayName = options.displayName;
    Object.assign(result, options.statics);
    // Later versions of TypeSript should allow us to merge objects in a type safe way and avoid this cast.
    return result;
}
/**
 * Evaluate styles based on type to return consistent TStyleSet.
 */
function _evaluateStyle(props, styles) {
    if (typeof styles === 'function') {
        return styles(props);
    }
    return styles;
}
//# sourceMappingURL=createComponent.js.map